# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:light
#     text_representation:
#       extension: .py
#       format_name: light
#       format_version: '1.5'
#       jupytext_version: 1.11.1
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# # Descriptive Statistics

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import matplotlib.ticker as mtick
import dabest

pd.set_option('display.max_rows', 3000)
pd.set_option('display.max_columns', None)
pd.options.display.float_format = '{:,.3f}'.format
sns.set(style='whitegrid', font_scale=1.2)
plt.rcParams['figure.figsize'] = [7, 5]

df = pd.read_csv("../data/endline.csv")

# ## Data cleaning

# Remove respondents who left the company in the month pre or post the experiment

df_excl = df[df.Post_Contr!=0].copy()

# Three respondents had retirement contributions rates below the minimum of 7.5 percent. These are likely "grandfathered" accounts that cannot be changed. At the time of randomisation, I was only shown bins of the retirement contribution rates, and these observations were in the 7.5 percent bin. Therefore, I will replace their observations with 7.5. 

df_excl.loc[df_excl.Pre_Contr<7.5,]

df_excl.loc[df_excl.Pre_Contr<7.5,"Pre_Contr"] = 7.5

df_excl.loc[df_excl.Post_Contr<7.5,"Post_Contr"] = 7.5

df_excl.describe()

# ## Stats in text

df.Spoke_on_phone.value_counts()

pd.crosstab(df.Spoke_on_phone,df.click_ind, margins=True)

pd.crosstab(df.Spoke_on_phone,df.click_ind,normalize='index')

df.Sentiment_towards_calculator.value_counts()

# ## Figures

# ### Histogram of pre-experiment retirement contribution rates

df_excl.Pre_Contr.describe()

np.arange(6.75,27.75,1)

len(df_excl)

fig = plt.figure(figsize=(12,4))
ax = df_excl.Pre_Contr.hist(bins=np.arange(6.99,27.99,1), color='#31a354', weights=np.ones(len(df_excl)) / len(df_excl))
ax.set(xlabel='Retirement Contribution Rate', ylabel='Share of Employees')
ax.xaxis.set_major_formatter(mtick.PercentFormatter(100, decimals=1))
plt.gca().yaxis.set_major_formatter(mtick.PercentFormatter(1))
fig.savefig('cont_hist.eps', bbox_inches='tight')

# ### Estimation plot

# Colors
# - control: #8da0cb
# - email: #31a354
# - phone: #006d2c

# Create an array with the colors you want to use
colors = ["#8da0cb", "#31a354","#006d2c"]
# Set your custom color palette
sns.set_palette(sns.color_palette(colors))

df_excl.head()

df_excl["Diff_Contr"] = df_excl.Post_Contr - df_excl.Pre_Contr

df_excl.reset_index(inplace=True)

est_plot = dabest.load(df_excl, idx=("Control","Email","Email and phone"), x="treatment_status", y="Diff_Contr") 

est_plot

est_plot.mean_diff

fig = est_plot.mean_diff.plot(fig_size=(10,8), swarm_label="\u0394 Contribution rate \n in percentage points")
fig.savefig("estplot.eps")
