# File:   Results.R
# Author: Matthew Olckers
# Date: 12 November 2019

#=========================================================================

# Installs pacman ("package manager") if needed
if (!require("pacman")) install.packages("pacman")



# Use pacman to load add-on packages
pacman::p_load(pacman, rio, rstudioapi, sandwich, Matrix, 
               ri2, randomizr, estimatr, plyr, dabestr,
               texreg)


# Change the working directory
set_wd <- function() {
  library(rstudioapi) # make sure you have it installed
  current_path <- getActiveDocumentContext()$path 
  setwd(dirname(current_path ))
  print( getwd() )
}
set_wd()

source("BMlmSE.R")

set.seed(1117192030) # Winning daily lotto numbers South Africa 18 August




#=========================================================================

df <- import("../data/endline.csv")

# Some data cleaning that could be moved
df$treatment_status[df$treatment_status=="Email and phone"] <- "Phone" 
count(df$treatment_status)

# Exclude employees who left the company in the month 
# before or after the experment.
df_excl <- df[!(df$Post_Contr==0),]

# Three employees have grandfathered accounts of less than
# 7.5 percent. At the time of randomisation, I was only given
# categories of baseline contribution rates and these employees
# were included in 7.5 percent bucket. I therefore convert their
# contribution rates to the minimum of 7.5. This adjustment does
# not lead to any noticeable change in the results. 
df_excl$Pre_Contr[df_excl$Pre_Contr<7.5] <- 7.5 
df_excl$Post_Contr[df_excl$Post_Contr<7.5] <- 7.5 

# Difference in contribution rates pre and post experiment
df_excl$Diff_Contr <- df_excl$Post_Contr - df_excl$Pre_Contr


#=========================================================================

# Estimate models

ols_level <- lm_robust(formula=Post_Contr~treatment_status,
                 data=df_excl, fixed_effects = ~ Group)

ols_diff <- lm_robust(formula=Diff_Contr~treatment_status,
                      data=df_excl, fixed_effects = ~ Group)


iv_level <- iv_robust(formula=Post_Contr~click_ind|treatment_status,
                      data=df_excl, fixed_effects = ~ Group)

iv_diff <- iv_robust(formula=Diff_Contr~click_ind|treatment_status,
                      data=df_excl, fixed_effects = ~ Group)



#=========================================================================

treatment <- factor(df_excl$treatment_status)
groups <- factor(df_excl$Group)
outcome <- df_excl$Post_Contr
outcome_diff <- df_excl$Post_Contr - df_excl$Pre_Contr



ols.fit <- lm(outcome ~ treatment + groups, singular.ok = FALSE)
ols.fit_diff <- lm(outcome_diff ~ treatment + groups, singular.ok = FALSE)

# Use BMlmSE() to compute the BM SE and degrees of freedom

ell.email = rep(0, length(ols.fit$coefficients))
ell.email_diff = rep(0, length(ols.fit_diff$coefficients))
ell.email[2] <- 1
ell.email_diff[2] <- 1

ell.phone = rep(0, length(ols.fit$coefficients))
ell.phone_diff = rep(0, length(ols.fit_diff$coefficients))
ell.phone[3] <- 1
ell.phone_diff[3] <- 1

bm.email <- BMlmSE(model = ols.fit, ell = ell.email)
bm.phone <- BMlmSE(model = ols.fit, ell = ell.phone)
bm.email_diff <- BMlmSE(model = ols.fit_diff, ell = ell.email_diff)
bm.phone_diff <- BMlmSE(model = ols.fit_diff, ell = ell.phone_diff)

point.est.email <- coef(ols.fit)['treatmentEmail']
point.est.phone <- coef(ols.fit)['treatmentPhone']
point.est.email_diff <- coef(ols.fit_diff)['treatmentEmail']
point.est.phone_diff <- coef(ols.fit_diff)['treatmentPhone']

# Critical value for 95% CI
critical.value.email <- qt(0.975, df = bm.email$dof)  
critical.value.phone <- qt(0.975, df = bm.phone$dof)
critical.value.email_diff <- qt(0.975, df = bm.email_diff$dof)  
critical.value.phone_diff <- qt(0.975, df = bm.phone_diff$dof)

margin.of.error.email <- critical.value.email * bm.email$se
margin.of.error.phone <- critical.value.phone * bm.phone$se
margin.of.error.email_diff <- critical.value.email_diff * bm.email_diff$se
margin.of.error.phone_diff <- critical.value.phone_diff * bm.phone_diff$se

ci.email = c(point.est.email - margin.of.error.email, 
             point.est.email + margin.of.error.email)
ci.phone = c(point.est.phone - margin.of.error.phone, 
             point.est.phone + margin.of.error.phone)
ci.email_diff = c(point.est.email_diff - margin.of.error.email_diff, 
                  point.est.email_diff + margin.of.error.email_diff)
ci.phone_diff = c(point.est.phone_diff - margin.of.error.phone_diff, 
                  point.est.phone_diff + margin.of.error.phone_diff)
names(ci.email) = c('lower','upper')
names(ci.phone) = c('lower','upper')
names(ci.email_diff) = c('lower','upper')
names(ci.phone_diff) = c('lower','upper')

point.est.email  # Estimated average treatment effect
bm.email$se           # HC2 robust SE
bm.email$dof          # Bell-McCaffrey degrees of freedom
ci.email              # Bell-McCaffrey confidence interval

point.est.phone  # Estimated average treatment effect
bm.phone$se           # HC2 robust SE
bm.phone$dof          # Bell-McCaffrey degrees of freedom
ci.phone              # Bell-McCaffrey confidence interval

point.est.email_diff  # Estimated average treatment effect
bm.email_diff$se           # HC2 robust SE
bm.email_diff$dof          # Bell-McCaffrey degrees of freedom
ci.email_diff              # Bell-McCaffrey confidence interval

point.est.phone_diff  # Estimated average treatment effect
bm.phone_diff$se           # HC2 robust SE
bm.phone_diff$dof          # Bell-McCaffrey degrees of freedom
ci.phone_diff              # Bell-McCaffrey confidence interval


#======================================================================

# Randomization inference

# Declare randomization procedure
count(df_excl$treatment_status)

count(treatment)

N <- 765

declaration <- declare_ra(blocks = groups, num_arms = 3, 
                          conditions = c("Control", "Email", "Phone"),
                          prob_each = c(.5, .25, .25),
                          N = N)

Z <- treatment
Y <- outcome


dat <- data.frame(Y, Z, groups)

ri2_out <-
  conduct_ri(
    model_1 = Y ~ 1 + groups, # restricted model
    model_2 = Y ~ Z + groups, # unrestricted model
    declaration = declaration,
    sharp_hypothesis = 0,
    data = dat
  )

plot(ri2_out)

summary(ri2_out)

# for comparison
anova(lm(Y ~ 1 + groups, data = dat), 
      lm(Y ~ Z + groups, data = dat))

#=======================================================================

# Simple Means for Prediction Plot

aggregate(df_excl[, c("Post_Contr")], list(df_excl$treatment_status), mean)

se <- function(x) sqrt(var(x)/length(x))

aggregate(df_excl[, c("Post_Contr")], list(df_excl$treatment_status), se)

aggregate(df_excl[, c("Diff_Contr")], list(df_excl$treatment_status), mean)

#=======================================================================

# Local Average Treatment Effect

iv <- ivreg(Post_Contr~click_ind+groups|treatment_status+groups,data=df_excl)

iv_diff <- ivreg(Diff_Contr~click_ind|treatment_status+groups,data=df_excl)

iv$coefficients

ell.click = rep(0, length(iv$coefficients))
ell.click_diff = rep(0, length(iv_diff$coefficients))
ell.click[2] <- 1
ell.click_diff[2] <- 1


bm.click <- BMlmSE(model = iv, ell = ell.click)
bm.click_diff <- BMlmSE(model = iv_diff, ell = ell.click_diff)





help(ivreg)


#=======================================================================

# Export Results


ols.se <- c(bm.email$se,bm.phone$se)
names(ols.se) = c('treatmentEmail','treatmentPhone')


ols_diff.se <- c(bm.email_diff$se,bm.phone_diff$se)
names(ols_diff.se ) = c('treatmentEmail','treatmentPhone')


stargazer(ols.fit,ols.fit_diff,iv,iv_diff,
          type='text', omit=c("group*","Constant"),
          style="aer",
          se = list(ols.se,ols_diff.se),
          keep.stat = c("n","rsq"),
          covariate.labels = c("Email only", "Email and phone","Click on link"),
          dep.var.labels = c("Post Conc Rate","Change Conc Rate"),
          add.lines = list(c("Control Mean","9.010","0.066","9.010","0.066"),
                          c("First stage F-statistic","","")),
          notes        = "Robust standard errors in parenthesis. All specifications control for strata fixed effects.", 
          notes.append = FALSE)



help("stargazer")


#=======================================================================

# Local Average Treatment Effect

reg_ols1 <- lm(Post_Contr~treatment_status,data=df_excl)
reg_ols2 <- lm(Post_Contr~treatment_status + Pre_Contr,data=df_excl)

reg_iv1 <- ivreg(Post_Contr~click_ind|treatment_status,data=df_excl)
reg_iv2 <- ivreg(Post_Contr~Pre_Contr + click_ind|treatment_status+Pre_Contr,data=df_excl)
summary(reg_iv2)




#=======================================================================




# CLEAN UP #################################################

# Clear environment
rm(list = ls()) 

# Clear plots
dev.off()  # But only if there IS a plot

# Clear packages
p_unload(all) 
detach("package:datasets", unload = TRUE)  # For base

# Clear console
cat("\014")  # ctrl+L
